import boto3


def _get_client():
    from ridi.secret_keeper.config import AWS_CONNECT_ARGS

    client = boto3.client(
        'ssm',
        region_name=AWS_CONNECT_ARGS.aws_region,
        aws_access_key_id=AWS_CONNECT_ARGS.aws_access_key,
        aws_secret_access_key=AWS_CONNECT_ARGS.aws_secret_key,
    )
    return client


def tell(alias):
    response = _get_client().get_parameter(
        Name=alias, WithDecryption=True,
    )
    return response['Parameter']['Value']


def tell_safe(alias):
    try:
        return tell(alias)
    except Exception:
        return None


__all__ = ["tell", "tell_safe"]



import os

from collections import namedtuple

ENVNAME_AWS_ACCESS_KEY = "SECRETKEEPER_AWS_ACCESS_KEY"
ENVNAME_AWS_SECRET_KEY = "SECRETKEEPER_AWS_SECRET_KEY"
ENVNAME_AWS_REGION = "SECRETKEEPER_AWS_REGION"

ConnectArgsType = namedtuple('ConnectArgsType', [
    'aws_access_key', 'aws_secret_key', 'aws_region'
])

AWS_CONNECT_ARGS = None


def configure(aws_access_key=None, aws_secret_key=None, aws_region=None):
    global AWS_CONNECT_ARGS
    AWS_CONNECT_ARGS = ConnectArgsType(
        aws_access_key=aws_access_key,
        aws_secret_key=aws_secret_key,
        aws_region=aws_region,
    )


configure(
    aws_access_key=os.environ.get(ENVNAME_AWS_ACCESS_KEY),
    aws_secret_key=os.environ.get(ENVNAME_AWS_SECRET_KEY),
    aws_region=os.environ.get(ENVNAME_AWS_REGION),
)

__all__ = ["configure", "ENVNAME_AWS_ACCESS_KEY", "ENVNAME_AWS_SECRET_KEY", "ENVNAME_AWS_REGION"]



import os
import random
import string
import unittest
try:
    from StringIO import StringIO   # for Python2 support
except ImportError:
    from io import StringIO

from ridi.secret_keeper import tell, tell_safe
from ridi.secret_keeper.cmdline import run

try:
    from unittest import mock
except ImportError:
    import mock

from botocore.exceptions import ClientError


class FakeClient(object):
    def __init__(self, dict):
        self._dict = dict

    def get_parameter(self, Name, WithDecryption):
        assert WithDecryption is True
        if Name in self._dict:
            return {
                'Parameter': {
                    'Name': Name,
                    'Type': "SecureString",
                    'Value': self._dict[Name],
                }
            }
        else:
            error_response = {
                'Error': {
                    'Code': "ParameterNotFound",
                },
            }
            raise ClientError(error_response, operation_name="GetParameter")


class TestSecretKeeperBase(unittest.TestCase):
    def setUp(self):
        _client = FakeClient({
            "ones": "11111",
        })
        self._patcher = mock.patch('boto3.client', return_value=_client)
        self._patcher.start()

    def tearDown(self):
        self._patcher.stop()


class TestSecretKeeperTell(TestSecretKeeperBase):
    def test_existing_keys(self):
        self.assertEqual(tell("ones"), "11111")

    def test_non_existing_keys(self):
        with self.assertRaises(ClientError):
            tell("twos")


class TestSecretKeeperTellSafe(TestSecretKeeperBase):
    def test_existing_keys(self):
        self.assertEqual(tell_safe("ones"), "11111")

    def test_non_existing_keys(self):
        self.assertEqual(tell_safe("twos"), None)


class TestSecretKeeperCLI(TestSecretKeeperBase):
    def test_success_stdout(self):
        with mock.patch('sys.stdout', new=StringIO()) as fake_stdout:
            retval = run(["ones"])
            output = fake_stdout.getvalue().strip()
            self.assertEqual(output, "11111")
            self.assertEqual(retval, 0)

    def test_success_file(self):
        outfile = "tmp." + "".join(random.choice(string.ascii_letters) for _ in range(10))
        if os.path.exists(outfile):
            os.remove(outfile)

        retval = run(["ones", "--outfile", outfile])

        with open(outfile, "r") as f:
            content = f.read().strip()

        if os.path.exists(outfile):
            os.remove(outfile)

        self.assertEqual(content, "11111")
        self.assertEqual(retval, 0)

    def test_fail_stderr(self):
        with mock.patch('sys.stderr', new=StringIO()) as fake_stderr:
            retval = run(["twos"])
            output = fake_stderr.getvalue().strip()
            self.assertIn("Secret of alias 'twos' is not found.", output)
            self.assertEqual(retval, 1)



from __future__ import print_function
import argparse
import sys
import traceback

from botocore.exceptions import ClientError

from ridi.secret_keeper.connect import tell


def run(arguments):
    description = """
    Retrieve and print secrets from `secret-keeper`.
    You need to configure AWS credentials by environment variables or files.
    See https://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html#credentials for more detail.
    """
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument("alias", help="Alias of the secret")
    parser.add_argument("-o", "--outfile", help="Output file name. If not provided, secret is printed to stdout.")
    parser.add_argument("-v", "--verbose", action="store_true", help="Gives detailed error message")
    args = parser.parse_args(arguments)

    alias = args.alias
    outfile = args.outfile
    verbose = args.verbose

    try:
        secret = tell(alias).strip()
        if outfile:
            with open(outfile, "w") as f:
                print(secret, file=f)
        else:
            print(secret)
        return 0
    except Exception as e:
        if verbose:
            traceback.print_exc(file=sys.stderr)
        if isinstance(e, ClientError):
            print("Secret of alias '%s' is not found." % alias, file=sys.stderr)
        return 1


def main():
    retval = run(sys.argv[1:])
    sys.exit(retval)


if __name__ == "__main__":
    main()




"""A setuptools based setup module.
"""

from setuptools import setup, find_packages

setup(
    name='secret-keeper',
    packages=[
        'ridi.secret_keeper',
    ],
    version='0.3.1',
    description='Secret Keeper',
    url='https://github.com/ridi/secret-keeper-python',
    keywords=['secret', 'secret-keeper', 'ridi', 'ridibooks'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=[
        'boto3>=1.9.16',
    ],
    entry_points={
        'console_scripts': [
            'secret-keeper = ridi.secret_keeper.cmdline:main'
        ]
    },
)
